@@ -16,7 +16,7 @@ import operator
 import time
 import logging
 
-log = logging.getLogger('client')
+log = logging.getLogger('vncdotool.client')
 
 
 KEYMAP = {
@@ -123,6 +123,9 @@ class VNCDoToolClient(rfb.RFBClient):
     cmask = None
 
     def _decodeKey(self, key):
+        if self.factory.force_caps and key.isupper():
+            key = 'shift-%c' % key
+
         if len(key) == 1:
             keys = [key]
         else:
@@ -170,7 +173,7 @@ class VNCDoToolClient(rfb.RFBClient):
             button: int: [1-n]
 
         """
-        log.debug('mousePress', button)
+        log.debug('mousePress %s', button)
         buttons = self.buttons | (1 << (button - 1))
         self.pointerEvent(self.x, self.y, buttonmask=buttons)
         self.pointerEvent(self.x, self.y, buttonmask=self.buttons)
@@ -183,7 +186,7 @@ class VNCDoToolClient(rfb.RFBClient):
             button: int: [1-n]
 
         """
-        log.debug('mouseDown', button)
+        log.debug('mouseDown %s', button)
         self.buttons |= 1 << (button - 1)
         self.pointerEvent(self.x, self.y, buttonmask=self.buttons)
 
@@ -195,7 +198,7 @@ class VNCDoToolClient(rfb.RFBClient):
             button: int: [1-n]
 
         """
-        log.debug('mouseUp', button)
+        log.debug('mouseUp %s', button)
         self.buttons &= ~(1 << (button - 1))
         self.pointerEvent(self.x, self.y, buttonmask=self.buttons)
 
@@ -204,16 +207,28 @@ class VNCDoToolClient(rfb.RFBClient):
     def captureScreen(self, filename):
         """ Save the current display to filename
         """
-        # request screen update
-        log.debug('captureScreen', filename)
+        log.debug('captureScreen %s', filename)
+        return self._capture(filename)
+
+    def captureRegion(self, filename, x, y, w, h):
+        """ Save a region of the current display to filename
+        """
+        log.debug('captureRegion %s', filename)
+        return self._capture(filename, x, y, x+w, y+h)
+
+    def _capture(self, filename, *args):
         self.framebufferUpdateRequest()
         self.deferred = Deferred()
-        self.deferred.addCallback(self._captureSave, filename)
+        self.deferred.addCallback(self._captureSave, filename, *args)
         return self.deferred
 
-    def _captureSave(self, data, filename):
-        log.debug('captureDone', filename)
-        self.screen.save(filename)
+    def _captureSave(self, data, filename, *args):
+        log.debug('captureSave %s', filename)
+        if args:
+            capture = self.screen.crop(args)
+        else:
+            capture = self.screen
+        capture.save(filename)
 
         return self
 
@@ -224,27 +239,43 @@ class VNCDoToolClient(rfb.RFBClient):
             maxrms: the maximum root mean square between histograms of the
                     screen and target image
         """
-        log.debug('expectScreen', filename)
+        log.debug('expectScreen %s', filename)
+        return self._expectFramebuffer(filename, 0, 0, maxrms)
+
+    def expectRegion(self, filename, x, y, maxrms=0):
+        """ Wait until a portion of the screen matches the target image
+
+            The region compared is defined by the box
+            (x, y), (x + image.width, y + image.height)
+        """
+        log.debug('expectRegion %s (%s, %s)', filename, x, y)
+        return self._expectFramebuffer(filename, x, y, maxrms)
+
+    def _expectFramebuffer(self, filename, x, y, maxrms):
         self.framebufferUpdateRequest()
-        self.expected = ImageFactory().open(filename).histogram()
+        image = ImageFactory().open(filename)
+        w, h = image.size
+        self.expected = image.histogram()
         self.deferred = Deferred()
-        self.deferred.addCallback(self._expectCompare, maxrms)
+        self.deferred.addCallback(self._expectCompare, (x, y, x+w, y+h), maxrms)
 
         return self.deferred
 
-    def _expectCompare(self, image, maxrms):
+    def _expectCompare(self, image, box, maxrms):
+        image = image.crop(box)
+
         hist = image.histogram()
         if len(hist) == len(self.expected):
-            rms = math.sqrt(
-                        reduce(operator.add, map(lambda a, b: (a - b) ** 2,
-                            hist, self.expected)) / len(hist))
+            rms = math.sqrt(reduce(operator.add,
+                                   map(lambda a, b: (a - b) ** 2,
+                                   hist, self.expected)) / len(hist))
 
-            log.debug('rms', int(rms))
+            log.debug('rms %s', int(rms))
             if rms <= maxrms:
                 return self
 
         self.deferred = Deferred()
-        self.deferred.addCallback(self._expectCompare, maxrms)
+        self.deferred.addCallback(self._expectCompare, box, maxrms)
         self.framebufferUpdateRequest(incremental=1)
 
         return self.deferred
@@ -252,7 +283,7 @@ class VNCDoToolClient(rfb.RFBClient):
     def mouseMove(self, x, y):
         """ Move the mouse pointer to position (x, y)
         """
-        log.debug('mouseMove', x, y)
+        log.debug('mouseMove %d,%d', x, y)
         self.x, self.y = x, y
         self.pointerEvent(x, y, self.buttons)
         return self
@@ -260,7 +291,7 @@ class VNCDoToolClient(rfb.RFBClient):
     def mouseDrag(self, x, y, step=1):
         """ Move the mouse point to position (x, y) in increments of step
         """
-        log.debug('mouseDrag', x, y)
+        log.debug('mouseDrag %d,%d', x, y)
         if x < self.x:
             xsteps = [self.x - i for i in xrange(step, self.x - x + 1, step)]
         else:
@@ -313,7 +344,7 @@ class VNCDoToolClient(rfb.RFBClient):
         update = ImageFactory().fromstring('RGB', size, data, 'raw', 'RGBX')
         if not self.screen:
             self.screen = update
-        # track screen upward screen resizes, often occur during os boot
+        # track upward screen resizes, often occurs during os boot of VMs
         elif self.screen.size[0] < width or self.screen.size[1] < height:
             self.screen = update
         else:
@@ -365,14 +396,13 @@ class VNCDoToolFactory(rfb.RFBFactory):
 
     pseudocusor = False
     nocursor = False
+    force_caps = False
 
     def __init__(self):
         self.deferred = Deferred()
 
     def clientConnectionFailed(self, connector, reason):
         self.deferred.errback(reason)
-        self.deferred = None
 
     def clientConnectionMade(self, protocol):
         self.deferred.callback(protocol)
-        self.deferred = None
